@@ -142,7 +142,7 @@ class ContentType(models.Model):
         unique_together = (('app_label', 'model'),)
 
     def __str__(self):
-        return self.name
+        return self.app_labeled_name
 
     @property
     def name(self):
@@ -151,6 +151,13 @@ class ContentType(models.Model):
             return self.model
         return str(model._meta.verbose_name)
 
+    @property
+    def app_labeled_name(self):
+        model = self.model_class()
+        if not model:
+            return self.model
+        return '%s | %s' % (model._meta.app_label, model._meta.verbose_name)
+
     def model_class(self):
         """Return the model class for this type of content."""
         try:
