@@ -16,7 +16,7 @@ class JSONString(str):
 
 class JSONField(forms.CharField):
     default_error_messages = {
-        'invalid': _("'%(value)s' value must be valid JSON."),
+        'invalid': _('“%(value)s” value must be valid JSON.'),
     }
     widget = forms.Textarea
 
@@ -29,7 +29,7 @@ class JSONField(forms.CharField):
             return value
         try:
             converted = json.loads(value)
-        except ValueError:
+        except json.JSONDecodeError:
             raise forms.ValidationError(
                 self.error_messages['invalid'],
                 code='invalid',
@@ -45,10 +45,18 @@ class JSONField(forms.CharField):
             return initial
         try:
             return json.loads(data)
-        except ValueError:
+        except json.JSONDecodeError:
             return InvalidJSONInput(data)
 
     def prepare_value(self, value):
         if isinstance(value, InvalidJSONInput):
             return value
         return json.dumps(value)
+
+    def has_changed(self, initial, data):
+        if super().has_changed(initial, data):
+            return True
+        # For purposes of seeing whether something has changed, True isn't the
+        # same as 1 and the order of keys doesn't matter.
+        data = self.to_python(data)
+        return json.dumps(initial, sort_keys=True) != json.dumps(data, sort_keys=True)
