@@ -29,15 +29,15 @@ class Command(BaseCommand):
             help='Specify the app label(s) to create migrations for.',
         )
         parser.add_argument(
-            '--dry-run', action='store_true', dest='dry_run',
+            '--dry-run', action='store_true',
             help="Just show what migrations would be made; don't actually write them.",
         )
         parser.add_argument(
-            '--merge', action='store_true', dest='merge',
+            '--merge', action='store_true',
             help="Enable fixing of migration conflicts.",
         )
         parser.add_argument(
-            '--empty', action='store_true', dest='empty',
+            '--empty', action='store_true',
             help="Create an empty migration.",
         )
         parser.add_argument(
@@ -45,9 +45,13 @@ class Command(BaseCommand):
             help='Tells Django to NOT prompt the user for input of any kind.',
         )
         parser.add_argument(
-            '-n', '--name', action='store', dest='name', default=None,
+            '-n', '--name',
             help="Use this name for migration file(s).",
         )
+        parser.add_argument(
+            '--no-header', action='store_false', dest='include_header',
+            help='Do not add header comments to new migration file(s).',
+        )
         parser.add_argument(
             '--check', action='store_true', dest='check_changes',
             help='Exit with a non-zero status if model changes are missing migrations.',
@@ -61,27 +65,21 @@ class Command(BaseCommand):
         self.merge = options['merge']
         self.empty = options['empty']
         self.migration_name = options['name']
+        if self.migration_name and not self.migration_name.isidentifier():
+            raise CommandError('The migration name must be a valid Python identifier.')
+        self.include_header = options['include_header']
         check_changes = options['check_changes']
 
         # Make sure the app they asked for exists
         app_labels = set(app_labels)
-        bad_app_labels = set()
+        has_bad_labels = False
         for app_label in app_labels:
             try:
                 apps.get_app_config(app_label)
-            except LookupError:
-                bad_app_labels.add(app_label)
-        if bad_app_labels:
-            for app_label in bad_app_labels:
-                if '.' in app_label:
-                    self.stderr.write(
-                        "'%s' is not a valid app label. Did you mean '%s'?" % (
-                            app_label,
-                            app_label.split('.')[-1],
-                        )
-                    )
-                else:
-                    self.stderr.write("App '%s' could not be found. Is it in INSTALLED_APPS?" % app_label)
+            except LookupError as err:
+                self.stderr.write(str(err))
+                has_bad_labels = True
+        if has_bad_labels:
             sys.exit(2)
 
         # Load the current graph state. Pass in None for the connection so
@@ -195,7 +193,7 @@ class Command(BaseCommand):
                 self.stdout.write(self.style.MIGRATE_HEADING("Migrations for '%s':" % app_label) + "\n")
             for migration in app_migrations:
                 # Describe the migration
-                writer = MigrationWriter(migration)
+                writer = MigrationWriter(migration, self.include_header)
                 if self.verbosity >= 1:
                     # Display a relative path if it's below the current working
                     # directory, or an absolute path otherwise.
@@ -212,8 +210,7 @@ class Command(BaseCommand):
                     # Write the migrations file to the disk.
                     migrations_directory = os.path.dirname(writer.path)
                     if not directory_created.get(app_label):
-                        if not os.path.isdir(migrations_directory):
-                            os.mkdir(migrations_directory)
+                        os.makedirs(migrations_directory, exist_ok=True)
                         init_path = os.path.join(migrations_directory, "__init__.py")
                         if not os.path.isfile(init_path):
                             open(init_path, "w").close()
@@ -295,7 +292,7 @@ class Command(BaseCommand):
                     self.migration_name or ("merge_%s" % get_migration_name_timestamp())
                 )
                 new_migration = subclass(migration_name, app_label)
-                writer = MigrationWriter(new_migration)
+                writer = MigrationWriter(new_migration, self.include_header)
 
                 if not self.dry_run:
                     # Write the merge migrations file to the disk
