@@ -1,10 +1,11 @@
-import contextlib
 import copy
 from decimal import Decimal
 
 from django.apps.registry import Apps
 from django.db.backends.base.schema import BaseDatabaseSchemaEditor
 from django.db.backends.ddl_references import Statement
+from django.db.backends.utils import strip_quotes
+from django.db.models import UniqueConstraint
 from django.db.transaction import atomic
 from django.db.utils import NotSupportedError
 
@@ -19,11 +20,19 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
 
     def __enter__(self):
         # Some SQLite schema alterations need foreign key constraints to be
-        # disabled. Enforce it here for the duration of the transaction.
-        self.connection.disable_constraint_checking()
+        # disabled. Enforce it here for the duration of the schema edition.
+        if not self.connection.disable_constraint_checking():
+            raise NotSupportedError(
+                'SQLite schema editor cannot be used while foreign key '
+                'constraint checks are enabled. Make sure to disable them '
+                'before entering a transaction.atomic() context because '
+                'SQLite does not support disabling them in the middle of '
+                'a multi-statement transaction.'
+            )
         return super().__enter__()
 
     def __exit__(self, exc_type, exc_value, traceback):
+        self.connection.check_constraints()
         super().__exit__(exc_type, exc_value, traceback)
         self.connection.enable_constraint_checking()
 
@@ -75,11 +84,12 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         return False
 
     def alter_db_table(self, model, old_db_table, new_db_table, disable_constraints=True):
-        if disable_constraints and self._is_referenced_by_fk_constraint(old_db_table):
+        if (not self.connection.features.supports_atomic_references_rename and
+                disable_constraints and self._is_referenced_by_fk_constraint(old_db_table)):
             if self.connection.in_atomic_block:
                 raise NotSupportedError((
                     'Renaming the %r table while in a transaction is not '
-                    'supported on SQLite because it would break referential '
+                    'supported on SQLite < 3.26 because it would break referential '
                     'integrity. Try adding `atomic = False` to the Migration class.'
                 ) % old_db_table)
             self.connection.enable_constraint_checking()
@@ -93,11 +103,12 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         table_name = model._meta.db_table
         _, old_column_name = old_field.get_attname_column()
         if (new_field.name != old_field_name and
+                not self.connection.features.supports_atomic_references_rename and
                 self._is_referenced_by_fk_constraint(table_name, old_column_name, ignore_self=True)):
             if self.connection.in_atomic_block:
                 raise NotSupportedError((
                     'Renaming the %r.%r column while in a transaction is not '
-                    'supported on SQLite because it would break referential '
+                    'supported on SQLite < 3.26 because it would break referential '
                     'integrity. Try adding `atomic = False` to the Migration class.'
                 ) % (model._meta.db_table, old_field_name))
             with atomic(self.connection.alias):
@@ -130,11 +141,17 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         """
         Shortcut to transform a model from old_model into new_model
 
+        This follows the correct procedure to perform non-rename or column
+        addition operations based on SQLite's documentation
+
+        https://www.sqlite.org/lang_altertable.html#caution
+
         The essential steps are:
-          1. rename the model's existing table, e.g. "app_model" to "app_model__old"
-          2. create a table with the updated definition called "app_model"
-          3. copy the data from the old renamed table to the new table
-          4. delete the "app_model__old" table
+          1. Create a table with the updated definition called "new__app_model"
+          2. Copy the data from the existing "app_model" table to the new table
+          3. Drop the "app_model" table
+          4. Rename the "new__app_model" table to "app_model"
+          5. Restore any index of the previous "app_model" table.
         """
         # Self-referential fields must be recreated rather than copied from
         # the old model to ensure their remote_field.field_name doesn't refer
@@ -196,11 +213,6 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         # Work inside a new app registry
         apps = Apps()
 
-        # Provide isolated instances of the fields to the new model body so
-        # that the existing model's internals aren't interfered with when
-        # the dummy model is constructed.
-        body = copy.deepcopy(body)
-
         # Work out the new value of unique_together, taking renames into
         # account
         unique_together = [
@@ -222,51 +234,67 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                 if delete_field.name not in index.fields
             ]
 
-        # Construct a new model for the new state
+        constraints = list(model._meta.constraints)
+
+        # Provide isolated instances of the fields to the new model body so
+        # that the existing model's internals aren't interfered with when
+        # the dummy model is constructed.
+        body_copy = copy.deepcopy(body)
+
+        # Construct a new model with the new fields to allow self referential
+        # primary key to resolve to. This model won't ever be materialized as a
+        # table and solely exists for foreign key reference resolution purposes.
+        # This wouldn't be required if the schema editor was operating on model
+        # states instead of rendered models.
         meta_contents = {
             'app_label': model._meta.app_label,
             'db_table': model._meta.db_table,
             'unique_together': unique_together,
             'index_together': index_together,
             'indexes': indexes,
+            'constraints': constraints,
             'apps': apps,
         }
         meta = type("Meta", (), meta_contents)
-        body['Meta'] = meta
-        body['__module__'] = model.__module__
+        body_copy['Meta'] = meta
+        body_copy['__module__'] = model.__module__
+        type(model._meta.object_name, model.__bases__, body_copy)
 
-        temp_model = type(model._meta.object_name, model.__bases__, body)
+        # Construct a model with a renamed table name.
+        body_copy = copy.deepcopy(body)
+        meta_contents = {
+            'app_label': model._meta.app_label,
+            'db_table': 'new__%s' % strip_quotes(model._meta.db_table),
+            'unique_together': unique_together,
+            'index_together': index_together,
+            'indexes': indexes,
+            'constraints': constraints,
+            'apps': apps,
+        }
+        meta = type("Meta", (), meta_contents)
+        body_copy['Meta'] = meta
+        body_copy['__module__'] = model.__module__
+        new_model = type('New%s' % model._meta.object_name, model.__bases__, body_copy)
 
-        # We need to modify model._meta.db_table, but everything explodes
-        # if the change isn't reversed before the end of this method. This
-        # context manager helps us avoid that situation.
-        @contextlib.contextmanager
-        def altered_table_name(model, temporary_table_name):
-            original_table_name = model._meta.db_table
-            model._meta.db_table = temporary_table_name
-            yield
-            model._meta.db_table = original_table_name
+        # Create a new table with the updated schema.
+        self.create_model(new_model)
 
-        with altered_table_name(model, model._meta.db_table + "__old"):
-            # Rename the old table to make way for the new
-            self.alter_db_table(
-                model, temp_model._meta.db_table, model._meta.db_table,
-                disable_constraints=False,
-            )
-            # Create a new table with the updated schema.
-            self.create_model(temp_model)
+        # Copy data from the old table into the new table
+        self.execute("INSERT INTO %s (%s) SELECT %s FROM %s" % (
+            self.quote_name(new_model._meta.db_table),
+            ', '.join(self.quote_name(x) for x in mapping),
+            ', '.join(mapping.values()),
+            self.quote_name(model._meta.db_table),
+        ))
 
-            # Copy data from the old table into the new table
-            field_maps = list(mapping.items())
-            self.execute("INSERT INTO %s (%s) SELECT %s FROM %s" % (
-                self.quote_name(temp_model._meta.db_table),
-                ', '.join(self.quote_name(x) for x, y in field_maps),
-                ', '.join(y for x, y in field_maps),
-                self.quote_name(model._meta.db_table),
-            ))
+        # Delete the old table to make way for the new
+        self.delete_model(model, handle_autom2m=False)
 
-            # Delete the old table
-            self.delete_model(model, handle_autom2m=False)
+        # Rename the new table to take way for the old
+        self.alter_db_table(
+            new_model, new_model._meta.db_table, model._meta.db_table,
+            disable_constraints=False,
+        )
 
         # Run deferred SQL on correct table
         for sql in self.deferred_sql:
@@ -320,6 +348,14 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
     def _alter_field(self, model, old_field, new_field, old_type, new_type,
                      old_db_params, new_db_params, strict=False):
         """Perform a "physical" (non-ManyToMany) field update."""
+        # Use "ALTER TABLE ... RENAME COLUMN" if only the column name
+        # changed and there aren't any constraints.
+        if (self.connection.features.can_alter_table_rename_column and
+            old_field.column != new_field.column and
+            self.column_sql(model, old_field) == self.column_sql(model, new_field) and
+            not (old_field.remote_field and old_field.db_constraint or
+                 new_field.remote_field and new_field.db_constraint)):
+            return self.execute(self._rename_field_sql(model._meta.db_table, old_field, new_field, new_type))
         # Alter by remaking table
         self._remake_table(model, alter_field=(old_field, new_field))
         # Rebuild tables with FKs pointing to this field if the PK type changed.
@@ -362,3 +398,15 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         ))
         # Delete the old through table
         self.delete_model(old_field.remote_field.through)
+
+    def add_constraint(self, model, constraint):
+        if isinstance(constraint, UniqueConstraint) and constraint.condition:
+            super().add_constraint(model, constraint)
+        else:
+            self._remake_table(model)
+
+    def remove_constraint(self, model, constraint):
+        if isinstance(constraint, UniqueConstraint) and constraint.condition:
+            super().remove_constraint(model, constraint)
+        else:
+            self._remake_table(model)
